
#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1 Gold
 
//---- input parameters
extern string    TimeFrame   = "Current time frame";
extern string    ____0____   = "Period of average";
extern int       MAPeriod    = 50;
extern string    ____1____   = "Type of average: SMA - 0, EMA - 1, SMMA - 2, LWMA - 3";
extern int       MaType      = 1;
extern string    ____2____   = "Type of price:cl=0,op=1,hi=2,lo=3,med=4";
extern int       PriceType   = 4;
extern string    ____3____   = "Moving Average Shift";
extern int       MaShift     = 0;
extern string    ____4____   = "Show oversold and overbought levels";
extern bool      ShowLevels  = true;
extern string    ____5____   = "Lines color (60%)";
extern color     LinesColor1 = Lime;
extern string    ____6____   = "Lines color (100%)";
extern color     LinesColor2 = Lime;
extern string    ____7____   = "Lines style: solid - 0, dashed - 1, dotted - 2";
extern int       LinesStyle  = 2;

//---- buffers
double ExtMapBuffer1[];
int    timeFrame;

int init()
{
   SetIndexBuffer(0,ExtMapBuffer1);
   SetLevelValue(0,0.0);
   timeFrame = stringToTimeFrame(TimeFrame);
   SetIndexShift(0,MaShift  * timeFrame/Period()); 
   IndicatorShortName(timeFrameToString(timeFrame)+" MA Distance ("+MAPeriod+", "+MaType+", "+PriceType+", "+MaShift+")");
   return(0);
  }
   
int deinit()
{
   ObjectDelete("dev1");
   ObjectDelete("dev2");
   ObjectDelete("dev3");
   ObjectDelete("dev4");
   
return(0);
}

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit=MathMax(Bars-counted_bars,3*timeFrame/Period());
         double pipMultiplier = MathPow(10,MathMod(Digits,2));

   //
   //
   //
   //
   //
   //

   for(int i=limit; i >= 0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
      ExtMapBuffer1[i] = (iMA(NULL,timeFrame,1,0,MODE_SMA,PriceType,y)-iMA(NULL,timeFrame,MAPeriod,MaShift,MaType,PriceType,y))/(pipMultiplier*Point);
      
   }
   
   if(ShowLevels == true)
   {
      int window = WindowFind(timeFrameToString(timeFrame)+" MA Distance ("+MAPeriod+", "+MaType+", "+PriceType+", "+MaShift+")");
      if(window == -1)
      {
         window = 1;
      }
      // counting stddev
      ArraySetAsSeries(ExtMapBuffer1, true);
      double dev = iStdDevOnArray(ExtMapBuffer1, Bars-counted_bars-1, 100,0,0,0);
      // lines creation
      ObjectCreate("dev1", OBJ_HLINE, window, TimeCurrent(), 1.2*dev);
      ObjectSetText("dev1", "+60%");
      ObjectSet("dev1", OBJPROP_COLOR, LinesColor1);
      ObjectSet("dev1", OBJPROP_STYLE, LinesStyle);
      
      ObjectCreate("dev2", OBJ_HLINE, window, TimeCurrent(), -1.8*dev); 
      ObjectSetText("dev2", "-60%");
      ObjectSet("dev2", OBJPROP_COLOR, LinesColor1);
      ObjectSet("dev2", OBJPROP_STYLE, LinesStyle);
      
      ObjectCreate("dev3", OBJ_HLINE, window, TimeCurrent(), 2*dev);
      ObjectSetText("dev3", "+100%");
      ObjectSet("dev3", OBJPROP_COLOR, LinesColor2);
      ObjectSet("dev3", OBJPROP_STYLE, LinesStyle);
      
      ObjectCreate("dev4", OBJ_HLINE, window, TimeCurrent(), -3*dev);  
      ObjectSetText("dev4", "-100%");
      ObjectSet("dev4", OBJPROP_COLOR, LinesColor2);
      ObjectSet("dev4", OBJPROP_STYLE, LinesStyle);      
   }
   return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}


